"""Python client to access Djaizz data & models."""


class DjaizzClient:   # pylint: disable=too-few-public-methods
    """Python client to access Djaizz data & models."""

    def get_prediction(self, *args, **kwargs):
        """Get prediction from Djaizz server."""
